@@ -189,8 +189,9 @@ class Function:
         self._precedences = {}
 
         # Keep track of whether any of the signatures contains a parametric
-        # type. This is a necessary performance optimisation.
-        self._parametric = False
+        # type with type determined at runtime by `plum.type_of` instead of `type`.
+        # This is a necessary performance optimisation.
+        self._runtime_type_of = False
 
         self._cache = {}
         self._owner = ptype(owner) if owner else None
@@ -200,8 +201,9 @@ class Function:
 
         # Copy metadata.
         self.__name__ = f.__name__
-        self.__doc__ = f.__doc__
+        self.__qualname__ = f.__qualname__
         self.__module__ = f.__module__
+        self.__doc__ = f.__doc__
 
     @property
     def methods(self):
@@ -281,7 +283,7 @@ class Function:
             self._resolved = []
             self._methods.clear()
             self._precedences.clear()
-            self._parametric = False
+            self._runtime_type_of = False
 
     def register(self, signature, f, precedence=0, return_type=object):
         """Register a method.
@@ -319,9 +321,10 @@ class Function:
             # Add to resolved registrations.
             self._resolved.append((signature, f, precedence, return_type))
 
-            # Check whether the signature contains a parametric type.
-            if any(t.parametric for t in signature.types):
-                self._parametric = True
+            # Check whether the signature contains a type which requires `type_of` at
+            # runtime.
+            if any(t.runtime_type_of for t in signature.types):
+                self._runtime_type_of = True
 
         if registered:
             self._pending = []
@@ -445,13 +448,13 @@ class Function:
 
     def __call__(self, *args, **kw_args):
         # First resolve pending registrations, because the value of
-        # `self._parametric` depends on it.
+        # `self._runtime_type_of` depends on it.
         if len(self._pending) != 0:
             self._resolve_pending_registrations()
 
         # Get types of arguments for signature. Only use `type_of` if
         # necessary, because it incurs a significant performance hit.
-        if not self._parametric:
+        if not self._runtime_type_of:
             sig_types = tuple([type(x) for x in args])
         else:
             sig_types = tuple([promised_type_of.resolve()(x) for x in args])
@@ -501,7 +504,7 @@ class Function:
     def __repr__(self):
         return (
             f"<function {self._f} with "
-            f"{len(self._pending) + len(self._resolved)} method(s)>"
+            f"{len(self._pending) + len(self._methods)} method(s)>"
         )
 
 
